'use strict';

var magooshHost          = appConfiguration.host;
var flashcardDeckSetName = appConfiguration.deckSetName;
var flashcardDeckSetId   = appConfiguration.deckSetId;
var flashcardDeckSetSlug = appConfiguration.deckSetSlug;
var mixpanelToken        = appConfiguration.mixpanelToken;
var appVersion           = appConfiguration.appVersion;

var waitForAuth = true;

angular.module('magooshFlashcardApp', ['magooshServices', 'magooshFlashcardFilters'])
  .config(function ($routeProvider) {
    $routeProvider
      .when('/decks/', {
        templateUrl: 'views/decks.html',
        controller: 'DecksController'
      })
      .when('/decks/:deckId', {
        templateUrl: 'views/flashcard.html',
        controller: 'DeckController',
        reloadOnSearch: false
      })
      .when('/login', {
        templateUrl: 'views/login.html',
        controller: 'AccountController'
      })
      .when('/signup', {
        templateUrl: 'views/signup.html',
        controller: 'AccountController'
      })
      .when('/about', {
        templateUrl: 'views/about.html',
        controller: 'AboutController'
      })
      .when('/settings', {
        templateUrl: 'views/settings.html',
        controller: 'SettingsController'
      })
      .when('/feedback/:formName', {
        templateUrl: 'views/form-container.html',
        controller: 'FormController'
      })
      .otherwise({
        redirectTo: '/decks'
      });
  })
  
  // configuration for CORS
  .config(['$httpProvider', '$compileProvider', function($httpProvider, $compileProvider) {
      // Allow Cross-Origin Resource Sharing (CORS)
      $httpProvider.defaults.useXDomain = true;
      delete $httpProvider.defaults.headers.common['X-Requested-With'];

      // Allow custom URL schemes for deep app links
      $compileProvider.urlSanitizationWhitelist(/^\s*(https?|ftp|mailto|file|tel|content|market|itms):/);
  }])
  
  .run(['$rootScope', '$location', '$window', '$mixpanel', 'Auth', 'Deck', 'persistencejs', function($rootScope, $location, $window, $mixpanel, Auth, Deck, persistencejs){
    $rootScope.goBack = function(){
      gaRecordEvent("Button", "Click", "goBack", 1);
      $window.history.back();
    };
    
    // url should not contain "http://"
    $rootScope.openWebsite = function(url){
      // Override for loading urls in the browser instead of WebView
      window.open("http://" + url, '_system');
      return false;
    };
    
    $rootScope.backButton = function(event){
      gaRecordEvent("Hardware Button", "Press", "Back Button", 1);
      
      if (event) {event.preventDefault();}
      
      if (navigator && ($location.path() == '/decks'
                     || $location.path() == '/decks/')){
        console.log("Exiting app...");
        navigator.app.exitApp(); // This will exit the application
      } else {
        $window.history.back(); // This will return to the previous screen
      }
    };
    
    $rootScope.menuButton = function(event){
      gaRecordEvent("Hardware Button", "Press", "Menu Button", 1);
      
      $('.nav-collapse').collapse('toggle');
    }
    
    // Track changes in the url for Google Analytics
    $rootScope.$on("$routeChangeSuccess", function(event, current, previous){
      gaRecordPageLoad(magooshHost + '/flashcards/' + flashcardDeckSetSlug + $location.url());
    })

    $mixpanel.init(mixpanelToken, appVersion);

    var checkToken = function() {
      persistencejs.getAuthToken().then(function(token) {
        $mixpanel.setUserId(token);
        $mixpanel.trackAppLaunch();
      });
    };

    $rootScope.$on('Auth::Updated', checkToken);
    checkToken();

  }]);

/* Google Analytics */

var gaPlugin; // global gaPlugin variable
var gaPluginQueue = []; // queue to store events in before the plugin has been initialized

function nativePluginResultHandler (result) { console.log('nativePluginResultHandler: ' + result); }
function nativePluginErrorHandler  (error)  { console.log('nativePluginErrorHandler: '  + error ); }
// The app may be loaded before Google Analytics is initialized, so we must queue the tracking event
function gaRecordPageLoad (url) {
  if (gaPlugin) {
    gaPlugin.trackPage(nativePluginResultHandler, nativePluginErrorHandler, url);
  } else {
    gaPluginQueue.unshift(function(){
      gaRecordPageLoad(url);
    });
    console.log("Page load event queued: " + url);
  }
}
// category - This is the type of event you are sending such as "Button", "Menu", etc.
// eventAction - This is the type of event you are sending such as "Click", "Select". etc.
// eventLabel - A label that describes the event such as Button title or Menu Item name.
// eventValue - An application defined integer value that can mean whatever you want it to mean.
function gaRecordEvent(category, eventAction, eventLabel, eventValue){
  if (gaPlugin) {
    gaPlugin.trackEvent(nativePluginResultHandler, nativePluginErrorHandler, category, eventAction, eventLabel, eventValue);
  } else {
    gaPluginQueue.unshift(function(){
      gaRecordEvent(category, eventAction, eventLabel, eventValue);
    });
    console.log("Analytics event queued: " + category + ", " + eventAction + ", " + eventLabel + ", " + eventValue);
  }
}

function setupImageCache(){
  // Initialize the local image cache
  ImgCache.options.debug = true;
  ImgCache.init();
}
if (typeof(window.cordova) === 'undefined' && typeof(phonegap) === 'undefined'){
  setupImageCache(); // init ImgCache in Chrome
}

var onDeviceReady = function(){
  console.log("Fired onDeviceReady");
  
  // Button handling config from:
  // http://jnelson.in/angularjs/handling-android-device-back-button-with-phonegap-and-angular-js/jacob-nelson
  document.addEventListener("backbutton", handleDeviceBackButton, true);
  document.addEventListener("menubutton", handleDeviceMenuButton, false);
  
  gaPlugin = window.plugins.gaPlugin;
  gaPlugin.init(function(){}, function(){}, 'UA-9161165-16', 60); // success, error, ID, reporting period (in seconds)
  console.log("Initialized Google Analytics plugin");
  
  // Run any queued analytics statements
  var eventFunction;
  // console.log("Clearing " + gaPluginQueue.length + " events from Google Analytics queue...");
  while (eventFunction = gaPluginQueue.pop()){
    eventFunction();
  }
  // console.log("Finished clearing Google Analytics queue");
  
  setupImageCache();

}

function handleDeviceBackButton(event){
  angular.element('[ng-app=magooshFlashcardApp]').scope().backButton(event);
}

function handleDeviceMenuButton(event){
  angular.element('[ng-app=magooshFlashcardApp]').scope().menuButton(event);
}

document.addEventListener("deviceready", onDeviceReady, false);
