'use strict';

angular.module('magooshFlashcardApp').controller('DeckController', ['$scope', '$window', '$routeParams', '$mixpanel', 'Deck', function ($scope, $window, $routeParams, $mixpanel, Deck) {
  // initial state so that updateFlashcardContainerHeight has content
  $scope.deck = {}
  $scope.cards = [];
  
  var loadDeck = function(){
    return Deck.find($routeParams.deckId)
      .then(function(deck){
        $scope.deck = deck;
        return deck;
      }, function(){
        // Couldn't find the deck, return to list
        $window.history.back('/decks');
      });
  };
  
  // transition between two cards in $scope.cards
  var loadNewCard = function(newCard){
    console.log("loadNewCard");
    
    // add new card to the list
    $scope.cards.unshift(newCard);
    $scope.deck = newCard.deck;

    if ($scope.deck.mastered_cards_count == $scope.deck.cards_count) {
      $mixpanel.trackOnce('Deck Complete', {'Deck ID': $scope.deck.deck_id});
    }
    
    // Helper to remove all "out" cards from the list
    var cleanupOldCards = function(){
      $scope.$apply(function(){
        while($scope.cards[$scope.cards.length - 1].transition === "out") {
          $scope.cards.pop();
        }
      });
    };
    
    // start transition animation after submitResult() finishes
    setTimeout(function(){
      $scope.$apply(function(){
        // slide new card in
        console.log("Slide in card " + newCard.card_id);
        newCard.transition = "in";
        
        // trigger cleanup after fade finishes
        setTimeout(cleanupOldCards, 300);
      });
    }, 100);
  };
  
  // load and show the first card from the deck
  var loadFirstCard = function(deck){
    return Deck.pickCard(deck).then(loadNewCard);
  };
  
  $scope.submitCard = function(card, wasKnown){
    Deck.submitAnswerAndPickCard(card, wasKnown)
      .then(loadNewCard);
  };
  
  $(window).bind('resize', function(){
    $('.flashcard-container').updateFlashcardContainerHeight();
  });
  
  // Load deck
  loadDeck().then(loadFirstCard);
}]);
