'use strict';

angular.module('magooshFlashcardApp').directive('feedbackForm', ['persistencejs', '$location', function(persistencejs, $location){
  return {
    restrict: 'E',
    scope: {
      formName: '='
    },
    replace: true,
    templateUrl: "views/form.html",
    link: function($scope, $element, $attrs) {

      // Google Form Representation
      // - Version should be incremented whenever questions are changed.
      // - GoogleId keys are mappings to Google Form inputs.
      // - Update the showField method when changing questions to ensure proper questions are displayed.
      var formTemplates = {
        install: {
          url: 'https://docs.google.com/forms/d/1RQF94hb97njVI-7H0hgVcexsK8KQfLxTrDuariBFdVo/formResponse',
          version: 1,
          versionGoogleId: 'entry.589749564',
          tokenGoogleId: 'entry.1070716866',
          productGoogleId: 'entry.1324280010',
          headings: ["To provide a better experience for you, please answer these quick questions:"],
          questions: [
            {
              title: "Have you heard of Magoosh before?",
              name: "awareness",
              type: "radio",
              googleId: "entry.2000859529",
              options: [{text: 'Yes', value: 'Yes'}, {text: 'No', value: 'No'}]
            },
            {
              title: "Are you studying for an exam (TOEFL, GRE, etc.)?",
              name: "study",
              type: "radio",
              googleId: "entry.1530265491",
              options: [{text: 'Yes', value: 'Yes'}, {text: 'No', value: 'No'}]
            }
          ]
        },

        returnUser: {
          url: 'https://docs.google.com/forms/d/11KWQauKCvPFfpb-AQl4_28csinjK8uh1lUR6qLeFx0M/formResponse',
          version: 1,
          versionGoogleId: 'entry.1668300738',
          tokenGoogleId: 'entry.1048484191',
          productGoogleId: 'entry.553677292',
          questions: [
          {
            title: "How likely would you be to refer this app to a friend? (10 = very likely, 1 = not likely)",
            name: "refer",
            type: "select",
            googleId: "entry.1358614406",
            options: [{text: '', value: ''}, {text: '10', value: '10'}, {text: '9', value: '9'},
              {text: '8', value: '8'}, {text: '7', value: '7'}, {text: '6', value: '6'},
              {text: '5', value: '5'}, {text: '4', value: '4'}, {text: '3', value: '3'},
              {text: '2', value: '2'}, {text: '1', value: '1'}
            ]
          },
          {
            title: "How can we make this app better for you?",
            name: "improve",
            type: "textarea",
            googleId: "entry.111646838",
          },
          {
            title: "Please rate and review us in the app store! Better ratings help us get found by more people, which helps us make new free products!",
            type: "message"
          }
          ]
        }
      };

      $scope.formInfo = formTemplates[$scope.formName];
      $scope.score = null;

      if (navigator.userAgent.match(/iPad|iPhone|iPod/i)) {
        $scope.appStoreUrl = 'itms://itunes.apple.com/app/id' + appConfiguration.iosAppId;
      } else if (navigator.userAgent.match(/Android/i)) {
        $scope.appStoreUrl = 'market://details?id=' + appConfiguration.androidAppId;
      };

      // Input change callback for NPS question. This is only used to change form inputs on the
      // returnUser form.
      $scope.inputChange = function() {
        var score = angular.element('select[name=refer]').val();
        if (score == '') {
          return;
        } else {
          score = parseInt(score);
        }

        if (score > 7) {
          $scope.isPromoter = true;
          $scope.isDetractor = false;
        } else {
          $scope.isPromoter = false;
          $scope.isDetractor = true;
        }
      };

      // Determines if a question should be displayed. This can be used to display different
      // follow up questions
      $scope.showField = function(q) {
        if (q.type == 'textarea' && $scope.isDetractor) {
          return true;
        } else if (q.type == 'message' && $scope.isPromoter) {
          return true;
        } else if (q.type == 'radio' || q.type == 'select') {
          return true;
        } else {
          return false;
        }
      };

      // Extract form data and return it as a Javascript object
      var getFormData = function() {
        var data = {};

        var questions = $scope.formInfo.questions;
        for (var i = 0; i < questions.length; i++) {
          var q = questions[i];
          if (q.type == 'radio') {
            data[q.googleId] = $element.find('input[name=' + q.name + ']:checked').val();
          } else {
            data[q.googleId] = $element.find(q.type + '[name=' + q.name + ']').val();
          }
        }
        return data;
      };

      $scope.submit = function(){

        persistencejs.getAuthToken().then(function(token){
          var formData = getFormData();

          formData[$scope.formInfo.versionGoogleId] = $scope.formInfo.version;
          formData[$scope.formInfo.tokenGoogleId] = token;
          formData[$scope.formInfo.productGoogleId] = appConfiguration.deckSetName;

          $.ajax({
            url: $scope.formInfo.url,
            data: formData,
            type: 'POST',
            dataType: 'xml'
          });

          $location.path('/decks').replace();

        });
      };
    }
  };
}]);
