'use strict';

/*
 * Directive to handle interactions and transitions for flashcard elements.
 *
 * This directive expects the scope of the elemnt it's bound to to define a
 * `card` property:
 *
 *  <div class='flashcard' ng-repeat="card in cards" flashcard>
 *    <!-- ... -->
 *  </div>
 * 
 * The transition state of the card is controlled by the `card`'s `transition`
 * property. Set it to one of these values to slide the card to a position:
 * 
 *   "new" - off-screen to the right
 *   "in"  - on-screen
 *   "out" - off-screen to the left, then fade out (300ms)
 */

// TODO: figure out why new cards are being transitioned to "new" after "in"

angular.module('magooshFlashcardApp').directive('flashcard', [function(){
  return {
    restrict: 'A', // restrict to an attribute type
    link: function($scope, $element, attrs){
      $scope.card.transition = "new";
      
      // Make sure that the container height follows the card's height
      var refreshContainerHeight = function(){
        setTimeout(function(){
          $element.closest('.flashcard-container').updateFlashcardContainerHeight();
        }, 0);
      };
      // ...and initialize
      refreshContainerHeight();
      
      // Set the CSS style to match the card's transition state
      var renderTransitionState = function(){
        var state = $scope.card.transition;
        console.log('Moving card ' + $scope.card.card_id + ' to "' + state + '"');
        if (state === "new") {
          $element.css('transform', 'translateX(100%)');
        } else if (state === "in") {
          $element.css('transform', 'translateX(0%)');
        } else if (state === "out") {
          $element.css('transform', 'translateX(-125%)').fadeOut(300, refreshContainerHeight);
        }
      };
      
      // Initialize element's transition state
      renderTransitionState();
      // ...and watch for changes
      // http://stackoverflow.com/questions/20024156/how-to-watch-changes-on-models-created-by-ng-repeat
      $scope.$watch(function(){
        return $scope.card.transition;
      }, renderTransitionState);
      
      // Bind for flipping event
      $element.find('.front').bind('click', function(){
        console.log("Flip card " + $scope.card.card_id);
        $element.flipFlashcard();
        refreshContainerHeight();
        
        gaRecordEvent("Button", "Click", "flipCard", 1);
      });
      
      // Register the loading state and pass the event to FlashcardController
      $scope.answer = function(wasKnown) {
        console.log("Slide out card " + $scope.card.card_id);
        $element.find(wasKnown ? '.flashcard-btn.success' : '.flashcard-btn.danger').button('loading');
        $scope.card.transition = "out"; // slide out
        
        $scope.submitAnswer(wasKnown);
      };
    }
  };
}]);
