/*
 * This service is responsible for presenting an interface to controllers for
 * interacting with user credentials
 */

/* global magooshServices */

magooshServices.factory('Auth', ['$q', '$rootScope', 'persistencejs', 'FlashcardApi', function($q, $rootScope, persistencejs, FlashcardApi){
  
  return {
    /*
     * Create an account through the API and if it succeeds, then store the
     * user's auth data and broadcast "Auth::Updated" to the app
     * 
     * The returned promise is resolved if the request succeeds, or rejected
     * with an array of error messages if the request fails
     */
    createAccount: function(email, password){
      var deferred = $q.defer();
      
      FlashcardApi.login(email, password, true)
        .then(function(user){
          persistencejs.setUser(user)
            .then(function(){$rootScope.$broadcast('Auth::Updated');})
            .then(deferred.resolve, deferred.reject);
        }, deferred.reject);
      
      return deferred.promise;
    },
    
    /*
     * Send login credentials to the API and if it succeeds, then store the
     * user's auth data and broadcast "Auth::Updated" to the app
     * 
     * The returned promise is resolved if the request succeeds, or rejected
     * with an array of error messages if the request fails
     */
    login: function(email, password){
      var deferred = $q.defer();
      
      FlashcardApi.login(email, password, false)
        .then(function(user){
          persistencejs.setUser(user)
            .then(function(){$rootScope.$broadcast('Auth::Updated');})
            .then(deferred.resolve, deferred.reject);
        }, deferred.reject);
      
      return deferred.promise;
    },
    
    /*
     * Load the persisted user information
     * 
     * The returned promise is resolved with the user data (a Javascript object
     * with "email" and "token" attributes) if the user is logged in, and
     * rejected otherwise
     */
    user: function(){
      return persistencejs.getUser();
    },
    
    /*
     * Remove the stored user and deck data and broadcast "Auth::Updated" to
     * the app
     *
     * The returned promise is resolved after the data is deleted
     */
    logout: function(){
      return persistencejs.setUser(null)
        .then(function(){
          $rootScope.$broadcast('Auth::Updated');
        });
    }
  };
}]);
