/* 
 * This provider is responsible for presenting an interface with an analytics service.
 * Right now it is connected to Mixpanel.
 */

magooshServices.provider('$mixpanel', {
  $get: ['$rootScope', 'persistencejs',
    function($rootScope, persistencejs) {
      var deviceType;
      deviceType = function() {
        if (navigator.userAgent.match(/iPad|iPhone|iPod/i)) {
          return "ios";
        } else if (navigator.userAgent.match(/Android/i)) {
          return "android";
        } else {
          return "web";
        }
      };

      return {
        init: function(token, appVersion) {
          mixpanel.init(token);
          options = {
            'Platform': deviceType(),
            'App Version': appVersion
          }
          // These registered properties will be included with each event
          mixpanel.register(options);
        },
        setUserId: function(token) {
          // Identify users by Session Token
          mixpanel.identify(token);
        },
        alias: function(id, token) {
          // Associate the user's ID with their session token so that
          // events sent from our servers are correctly tracked (like subscription purchase events)
          mixpanel.alias(id, token);
        },
        track: function(name, options) {
          mixpanel.track(name, options);
        },
        trackOnce: function(name, options) {
          encodedName = 'tracked_' + name.toLowerCase().replace(/ /g, '_');
          persistencejs.getSessionData(encodedName).then(function(value) {
            if (!value) {
              mixpanel.track(name, options);
              persistencejs.setSessionData(encodedName, true);
            }
          });
        },
        trackAppLaunch: function() {
          persistencejs.getSessionData('tracked_install').then(function(installed) {
            if (installed) {
              mixpanel.track('Launch');
            } else {
              mixpanel.track('Install');
              persistencejs.setSessionData('tracked_install', true);
            }
          });
        }
      };

    }
  ]
});
